<?php
namespace App\Rclone;

use Symfony\Component\Process\Process;
use Symfony\Component\Process\Exception\ProcessFailedException;

use Exception;

class Rclone
{
    protected string $rclonePath;
    /**
     * Global options
     * 
     * @var array<string> 
     */
    protected array $globalOptions = [];

    protected string $version = "";

    function __construct(string $rclonePath = "rclone")
    {
        $this->rclonePath = $rclonePath;
        try
        {
            $version = $this->exec('--version');
            $this->version = explode("\n", $version)[0];
        }
        catch(ProcessFailedException $e)
        {
            throw new Exception("Check installation of rclone");
        }
    }

    function getVersion(): string
    {
        return $this->version;
    }

    function getSize(string $path): int
    {
        $output = $this->exec('size', ['--json', $path]);
        return (int)json_decode($output)->bytes;
    }

    function copy(string $src, string $dest, string $bandwidth = null): string
    {   
        $options = array();

        $options[] = $src;
        $options[] = $dest;
        if ($bandwidth) {
            $options[] = "--bwlimit";
            $options[] = $bandwidth;
        }

        return $this->exec('copy', $options);
    }

    /**
     * @param $command Top level Rclone command
     * @param array<String> $options
     */
    protected function exec(string $command, array $options = array()) : string
    {   
        $process = new Process(
            array_merge(
                [$this->rclonePath],
                $this->globalOptions,
                [$command],
                $options
            )
        );
        $process->setTimeout(4*3600);
        $process->run();

        // executes after the command finishes
        if (!$process->isSuccessful()) {
            throw new ProcessFailedException($process);
        }

        return $process->getOutput();
    }
}